#####################################################################
# This code is written and copyrighted by Clif Flynt, 1998
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Clif Flynt.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Flynt Consulting Services,
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@cflynt.com
#####################################################################
# stringx.tcl
#
# 	This file implements a set of procs for deciphering html pages.
#
#
#  stringx_SetDebug {value} - Sets the debug switch to value.  0 = no output
#
#  stringx_After {regExp text} - Returns portion of text after regExp
#
#  stringx_Before {regExp text} - Returns portion of text before regExp
#
#  stringx_Between {regExp1 regExp2 text} - Returns portion of text 
#					  between 
#
#  stringx_GetText {regExp text} - Returns the match of the regular expression.
#
#  stringx_CaselessGetText {regExp text}  - 
#  	     Returns the match of the regular expression with no case checking.
#
#  stringx_Sequence {actionlist text} - Extracts an item from a text based on 
#                                     a set of actions
#
#  stringx_FindAll { regexp txt} - returns a list of all the positions where
#		a regular expression occurs.
#
#  stringx_Count {regExp text} - returns the number of times that {regExp} 
#			    is matched in the text
#

if {[info exists argv] &&($argv == "-test")} {
    lappend auto_path "../Tools"
}


package provide stringx 1.0

    set stringxVarlist {stringxGlobal}
    set stringxGlobal(debugLevel) 0

    ##############################################################################
    #  stringx_SetDebug --
    #
    # Arguments
    #
    # Results:
    #

    proc stringx_SetDebug {value} {
	variable stringxGlobal
	set stringxGlobal(debugLevel) $value

    }

    ##############################################################################
    #  stringx_Debug --
    #
    # Arguments
    #
    # Results:
    #

    proc stringx_Debug {text} {
	variable stringxGlobal
	if {$stringxGlobal(debugLevel) > 0} {
	    puts $text
	}
    }

    ##############################################################################
    #  stringx_After --
    # 	Return the portion of the text after a regular expression match
    # Arguments
    #	regExp	- The regular expression
    #	text	- The text	
    # Results:
    #	Returns a subset of input text

    proc stringx_After {regExp text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	set rtn $text

	set x [regexp -indices -- $regExp $text m1]
	if {$x > 0} {
	    set pos [lindex $m1 0]
	    set rtn [string range $text $pos end]
	}
	return $rtn
    }

    ##############################################################################
    #  stringx_Before --
    # 	Return the portion of the text before a regular expression match
    #
    # Arguments
    #
    #	regExp	- The regular expression
    #	text	- The text	
    # Results:
    #	Returns a subset of input text
    #

    proc stringx_Before {regExp text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	set rtn $text

	set x [regexp -- $regExp $text m1]
	if {$x > 0} {
	    set pos [string first $m1 $text]
	    set pos [expr [string length $m1] + $pos -1]
	    set rtn [string range $text 0 $pos]
	}
	return $rtn
    }

    ##############################################################################
    #  stringx_Between --
    # 	Return the portion of the text between regular expression matches
    #
    # Arguments
    #
    #	reg1	- The first regular expression
    #	reg2	- The last regular expression
    #	text	- The text	
    # Results:
    #	Returns a subset of input text
    #
    proc stringx_Between {reg1 reg2 text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	regexp -indices $reg1 $text pos1
	regexp -indices $reg2 $text pos2

	if {$reg2 > $reg1} {
	    set txt [stringx_Before $reg2 $text]
	    stringx_Debug "stringx_Between: stringx_Before: $txt"
	    if {$txt != ""} {
		set txt [stringx_After $reg1 $txt]
		stringx_Debug "stringx_Between: stringx_After: $txt"
	    }
	} {
	    set txt [stringx_After $reg1 $text]
	    stringx_Debug "stringx_Between: stringx_After: $txt"
	    if {$txt != ""} {
		set txt [stringx_Before $reg2 $txt]
		stringx_Debug "stringx_Between: stringx_After: $txt"
	    }
	}
	return $txt
    }

    ##############################################################################
    #
    # stringx_GetText --
    # Return the match to a regular expression
    #
    # Arguments
    #
    #	reg	- The regular expression
    #	text	- The text	
    # Results:
    #	Returns a subset of input text
    #

    proc stringx_GetText {reg text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	set mat ""
	regexp -- $reg $text mat
	return $mat
    }
    ##############################################################################
    #
    # stringx_CaselessGetText --
    # Return the a caseless match to a regular expression
    #
    # Arguments
    #	reg	- The regular expression
    #	text	- The text	
    #
    # Results:
    #	Returns a subset of input text
    #

    proc stringx_CaselessGetText {reg text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"
	set mat ""
	regexp -nocase -- $reg $text mat
	return $mat
    }
    ##############################################################################
    #  stringx_Sequence --
    #  Perform a set of actions on a set of text
    #
    # Arguments
    #	lst - The list of Tcl commands
    #	text - The input text
    # Results:
    #	Returns a subset of input text
    #

    proc stringx_Sequence {lst text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	foreach l $lst {
	    append l " \$text"
	    stringx_Debug "TO EVAL: $l - $text"
	    set text [eval $l]
	}
	return $text
    }

    ##############################################################################
    #
    #  stringx_FindAll --
    # Return a list of positions where the regexp occurs.
    #
    # Arguments
    #	reg	- The regular expression
    #	text	- The text	
    # Results:
    #	Returns a subset of input text

    proc stringx_FindAll {reg text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	set off 0

	set l ""

	while {[set x [regexp -indices -- $reg $text m1]] == 1} {
	    set st [lindex $m1 0]
	    lappend l [expr $st + $off]
	    set trim [expr $st+1]
	    incr off $trim
	    set text [string range $text $trim end]
	    stringx_Debug "L: $l"
	}

	return $l
    }

    ##############################################################################
    #  stringx_Count --
    # 	Return the number of times regExp occurs in the text
    # Arguments
    #	regExp	- The regular expression
    #	text	- The text	
    # Results:
    #	Return the number of times regExp occurs in the text

    proc stringx_Count {regExp text} {
	variable stringxVarlist
	eval "variable $stringxVarlist"

	set x [regsub -all -nocase -- $regExp $text x dmy]
	return $x
    }


    ##############################################################################
    ##############################################################################
    #  stringx_validatetest --
    #
    # Arguments
    #
    # Results:
    #

    proc stringx_validatetest {expect got tst} {
	variable faillst
	if {![string match $expect $got]} {
	    stringx_Debug "FAILED $tst : EXPECTED: .$expect. - Got .$got."
	    lappend faillst $tst
	    return 1
	}
	return 0
    }

    ##############################################################################
    #  stringx_test --
    #
    # Arguments
    #
    # Results:
    #

    proc stringx_test {} {
	variable faillst

	set faillst ""

	set failcnt 0


	set text "one two three four five"

	#    stringx_Setstringx_Debug 1

	set t1 [stringx_After two $text]
	set expect "two three four five"
	incr failcnt [validatetest $expect $t1 "stringx_After(exact)"]

	set t1 [stringx_After {t[a-z]o} $text]
	incr failcnt [validatetest $expect $t1 "stringx_After(regexp)"]

	set t1 [stringx_Before two $text]
	set expect "one two"
	incr failcnt [validatetest $expect $t1 "stringx_Before(exact)"]

	set t1 [stringx_Before {t[^ ]+} $text]
	incr failcnt [validatetest $expect $t1 "stringx_Before(regexp)"]

	set t1 [stringx_Between two four $text]
	set expect "two three four"
	incr failcnt [validatetest $expect $t1 "stringx_Between(exact)"]

	set t1 [stringx_Between {t[^ ]+} {f[^i|^ ]+} $text]
	incr failcnt [validatetest $expect $t1 "stringx_Between(regexp)"]

	set evallst [list [list stringx_After two] [list stringx_Before {fi}] [list stringx_Between wo\
	  ur]]
	set t1 [stringx_Sequence $evallst $text]
	set expect "wo three four"
	incr failcnt [validatetest $expect $t1 "stringx_Sequence(exact)"]

	set evallst [list [list stringx_After t.o] [list stringx_Before {f[a-j]}]\
	  [list stringx_Between {[u-z][^ ]} {[aeiou]r}]]
	set t1 [stringx_Sequence $evallst $text]
	set expect "wo three four"
	incr failcnt [validatetest $expect $t1 "stringx_Sequence(regexp)"]

	set t1 [stringx_Count f $text]
	set expect "2"
	incr failcnt [validatetest $expect $t1 "stringx_Count(exact)"]

	set t1 [stringx_Count {[aeiou]} $text]
	set expect "9"
	incr failcnt [validatetest $expect $t1 "stringx_Count(regexp)"]

	set t1 [stringx_FindAll f $text]
	set expect {14 19}
	incr failcnt [validatetest $expect $t1 "Findall(exact-many)"]

	set t1 [stringx_FindAll one $text]
	set expect {0}
	incr failcnt [validatetest $expect $t1 "Findall(exact-1)"]

	if {$failcnt != 0} {
	    puts "FAILED $failcnt Tests: - $faillst"
	} {
	    puts "All primitive tests passed"
	}

    }

if {[info exists argv] &&($argv == "-test")} {
    stringx_test
}



